-- ToME - Tales of Middle-Earth
-- Copyright (C) 2009, 2010 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

local Stats = require "engine.interface.ActorStats"

newEffect{
	name = "SWEATING",
	desc = "I'm sweating all over",
	status = "detrimental",
   type = "physical",
	parameters = { power=1 },
	on_gain = function(self, err) return "I'm sweating real bad.", "+Sweats" end,
	on_lose = function(self, err) return "The sweats stop.", "-Sweats" end,
	activate = function(self, eff)
		eff.critid = self:addTemporaryValue("crit_chance", -10)
		eff.damid = self:addTemporaryValue("combat_dam", -3)
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("crit_chance", -10)
		self:removeTemporaryValue("combat_dam", -3)
	end,
}

newEffect{
	name = "SHAKING",
	desc = "I got the shakes real bad",
	status = "detrimental",
   type = "physical",
	parameters = { power=1 },
	on_gain = function(self, err) return "My hands shake uncontrollably.", "+Shakes" end,
	on_lose = function(self, err) return "The shakes stop.", "-Shakes" end,
	activate = function(self, eff)
		eff.critid = self:addTemporaryValue("crit_chance", -10)
      eff.aimid = self:addTemporaryValue("inc_stats", {
			[Stats.STAT_AIM] = -5,
			[Stats.STAT_STR] = -5,
		})
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("crit_chance", -10)
		self:removeTemporaryValue("inc_stats", {
			[Stats.STAT_AIM] = -5,
			[Stats.STAT_STR] = -5,
		})
	end,
}

newEffect{
	name = "RETCHING",
	desc = "My stomach is convulsing in pain",
	status = "detrimental",
   type = "physical",
	parameters = { power=1 },
	on_gain = function(self, err) return "My stomach convulses, and I double over retching in agony.", "+Retching" end,
	on_lose = function(self, err) return "The stomach pain stops.", "-Retching" end,
	activate = function(self, eff)
		eff.stamid = self:addTemporaryValue("max_stamina", -50)
      eff.aimid = self:addTemporaryValue("inc_stats", {
			[Stats.STAT_SPD] = -5,
		})
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("max_stamina", -50)
		self:removeTemporaryValue("inc_stats", {
			[Stats.STAT_SPD] = -5,
		})
	end,
}

newEffect{
	name = "DIZZY",
	desc = "My head is spinning and my vision is warped",
	status = "detrimental",
   type = "physical",
	parameters = { power=1 },
	on_gain = function(self, err) return "My head spins and my vision is warped.", "+Dizzy" end,
	on_lose = function(self, err) return "The stomach pain stops.", "-Dizzy" end,
	activate = function(self, eff)
		eff.stamid = self:addTemporaryValue("crit_defence", -20)
      eff.aimid = self:addTemporaryValue("inc_stats", {
			[Stats.STAT_AIM] = -8,
		})
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("crit_defence", -20)
		self:removeTemporaryValue("inc_stats", {
			[Stats.STAT_AIM] = -8,
		})
	end,
}

newEffect{
	name = "STUMBLING",
	desc = "I lose control of my feet",
	status = "detrimental",
   type = "physical",
	parameters = { power=1 },
	on_gain = function(self, err) return "I roughly stumble over the uneven floor.", "+Stumbling" end,
	on_lose = function(self, err) return "I regain control of my feet.", "-Stumbling" end,
	activate = function(self, eff)
      eff.aimid = self:addTemporaryValue("inc_stats", {
         [Stats.STAT_SPD] = -10,
			[Stats.STAT_AIM] = -5,
		})
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("inc_stats", {
         [Stats.STAT_SPD] = -10,
			[Stats.STAT_AIM] = -5,
		})
	end,
}

newEffect{
	name = "BLUR",
	desc = "My vision is blurred",
	status = "detrimental",
   type = "physical",
	parameters = { power=1 },
	on_gain = function(self, err) return "Everything goes foggy and blurred.", "+Blur" end,
	on_lose = function(self, err) return "My vision comes back into focus.", "-Blur" end,
	activate = function(self, eff)
      eff.aimid = self:addTemporaryValue("lite", -3)
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("lite", -3)
	end,
}

newEffect{
	name = "VOMITING",
	desc = "I can't hold back the sea of sick pouring from my stomach",
	status = "detrimental",
   type = "physical",
	parameters = { power=1 },
	on_gain = function(self, err) return "Black bile erupts down my lungs.", "+Vomiting" end,
	on_lose = function(self, err) return "I stop vomiting.", "-Vomiting" end,
	activate = function(self, eff)
		eff.stamid = self:addTemporaryValue("max_life", -15)
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("max_life", -15)
	end,
}